package net.sf.corn.converter.json;

import java.awt.RenderingHints;
import java.security.AuthProvider;
import java.security.Provider;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.IdentityHashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;
import java.util.WeakHashMap;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentSkipListMap;
import java.util.jar.Attributes;

import javax.management.openmbean.TabularDataSupport;
import javax.print.attribute.standard.PrinterStateReasons;
import javax.swing.UIDefaults;

import net.sf.corn.converter.ConversionException;
import net.sf.corn.converter.ObjectReferenceMap;
import static net.sf.corn.converter.json.IJsonConstants.*;
class CnvMap extends AbstractJsonConverter {

	@SuppressWarnings("restriction")
	CnvMap() {
		getConvertibles().add(Map.class);
		getConvertibles().add(HashMap.class);
		getConvertibles().add(TreeMap.class);
		getConvertibles().add(LinkedHashMap.class);
		getConvertibles().add(Attributes.class);
		getConvertibles().add(AuthProvider.class);
		getConvertibles().add(ConcurrentHashMap.class);
		getConvertibles().add(ConcurrentSkipListMap.class);
		getConvertibles().add(Hashtable.class);
		getConvertibles().add(Properties.class);
		getConvertibles().add(IdentityHashMap.class);
		getConvertibles().add(PrinterStateReasons.class);
		getConvertibles().add(Provider.class);
		getConvertibles().add(RenderingHints.class);
		getConvertibles().add(TabularDataSupport.class);
		getConvertibles().add(WeakHashMap.class);
		getConvertibles().add(UIDefaults.class);
		try{
			getConvertibles().add(javax.script.SimpleBindings.class);
		}catch(Exception e){
			
		}

	}

	@Override
	protected JsTypeObject inToTarget(Object o, ObjectReferenceMap referenceMap)
			throws ConversionException {
		JsTypeComplex jsonObject = new JsTypeComplex(o.getClass());
		Map<?, ?> map = (Map<?, ?>) o;
		JsTypeList mapdata = new JsTypeList();
		if (referenceMap == null)
			referenceMap = new ObjectReferenceMap();
		JsonConverter transformer = new JsonConverter();
		for (Object key : map.keySet()) {
			Object object = map.get(key);
			JsTypeComplex mapItem = new JsTypeComplex();
			mapItem.put(ATTR_MAP_KEY,
					transformer.toTargetType(key, referenceMap));

			mapItem.put(ATTR_MAP_VALUE,
					transformer.toTargetType(object, referenceMap));

			mapdata.add(mapItem);
		}
		jsonObject.put(ATTR_MAP, mapdata);
		return jsonObject;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	@Override
	protected Object inToJava(Class<?> clazz, JsTypeObject obj)
			throws ConversionException {
		if (!obj.isComplex()) {
			throw new ConversionException("Only list type is allowed");
		}
		JsTypeComplex jso = (JsTypeComplex) obj;
		try {
			JsTypeList jsonmap = (JsTypeList) jso.get(ATTR_MAP);
			if (jsonmap == null) {
				throw new ConversionException("map missing");
			}
			Map abmap = null;
			try {
				abmap = (Map) clazz.newInstance();
			} catch (InstantiationException e) {
				abmap = new HashMap<Object, Object>();
			}
			JsonConverter transformer = new JsonConverter();
			for (int i = 0; i < jsonmap.size(); i++) {
				JsTypeComplex mapItem = (JsTypeComplex) jsonmap.get(i);
				if (mapItem == null) {
					throw new ConversionException("map item (" + i + ") is missing");
				}
				abmap.put(transformer.toJava(mapItem.get(ATTR_MAP_KEY)),
						transformer.toJava(mapItem.get(ATTR_MAP_VALUE)));
			}
			return abmap;
		} catch (Exception e) {
			throw new ConversionException(e);
		}
	}

	@Override
	public boolean isConvertible(Class<?> clazz) {
		boolean ret = checkIfConvertibleFromRegisteredTypes(clazz);
		if (!ret) {
			ret = Map.class.isAssignableFrom(clazz);
		}
		return ret;
	}

	@Override
	public boolean isSubjectForCircularReferenceCheck() {
		return true;
	}
}
